import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;

import javax.swing.JFrame;
import javax.swing.JPanel;

public class UE7Aufgabe19Datei {

	// Ausgangsfeld
	static int[][] world = null;

	public static void main(String[] args) {
		init();
		JFrame frame = new JFrame("Game of Life");
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.setLayout(new BorderLayout());
		WorldPanelD panel = new WorldPanelD(UE7Aufgabe19Datei.world);
		panel.setPreferredSize(new Dimension(
				UE7Aufgabe19Datei.world[0].length * 20,
				UE7Aufgabe19Datei.world.length * 20));
		frame.add(panel, BorderLayout.CENTER);
		frame.pack();
		new SimulationD(panel).start();
		frame.setVisible(true);
	}

	static void init() {
		final String filename = IO.readString("Dateiname: ");
		char[][] input = IO.readFileAsCharMatrix(filename);
		world = new int[input.length][input[0].length];
		for (int r = 0; r < world.length; r++) {
			for (int c = 0; c < world[0].length; c++) {
				world[r][c] = input[r][c] - '0';
			}
		}
	}
}

class WorldPanelD extends JPanel {

	private int[][] world;

	public WorldPanelD(int[][] world) {
		this.world = world;
	}

	public int[][] getWorld() {
		return this.world;
	}

	public void setWorld(int[][] world) {
		this.world = world;
	}

	public void paintComponent(Graphics g) {
		super.paintComponent(g);
		for (int r = 0; r < this.world.length; r++) {
			for (int s = 0; s < this.world[r].length; s++) {
				if (this.world[r][s] == 0) {
					g.setColor(Color.WHITE);
				} else {
					g.setColor(Color.DARK_GRAY);
				}
				g.fillRect(s * 20, r * 20, 20, 20);
			}
		}
	}
}

class SimulationD extends Thread {

	WorldPanelD panel;

	public SimulationD(WorldPanelD panel) {
		this.panel = panel;
	}

	public void run() {
		try {
			Thread.sleep(1000);
			while (true) {
				this.panel.setWorld(GenerationsD.calcNextGeneration(this.panel
						.getWorld()));
				this.panel.repaint();
				Thread.sleep(500);
			}
		} catch (Exception exc) {
			exc.printStackTrace();
		}
	}
}

class GenerationsD {

	/**
	 * Liefert ein Feld mit der nchsten Generation gem der
	 * Game-of-Life-Regeln.
	 * 
	 * @param world
	 *            das aktuelle Feld; 1 entspricht: Zelle ist lebenbidg; 0
	 *            entspricht: Zelle ist tot
	 * @return ein Feld mit der nchsten Generation;1 entspricht: Zelle ist
	 *         lebenbidg; 0 entspricht: Zelle ist tot
	 */
	static int[][] calcNextGeneration(int[][] world) {
		int[][] newWorld = new int[world.length][world[0].length];
		for (int r = 0; r < world.length; r++) {
			for (int s = 0; s < world[r].length; s++) {
				int nachbarn = calcNachbarn(world, r, s);
				if (world[r][s] == 1) {
					if (nachbarn < 2 || nachbarn > 3) {
						newWorld[r][s] = 0;
					} else {
						newWorld[r][s] = 1;
					}
				} else { // tot
					if (nachbarn == 3) {
						newWorld[r][s] = 1;
					} else {
						newWorld[r][s] = 0;
					}
				}
			}
		}
		return newWorld;
	}

	static int calcNachbarn(int[][] world, int reihe, int spalte) {
		int nachbarn = 0;
		for (int s = spalte - 1; s <= spalte + 1; s++) {
			if (world[(reihe - 1 + world.length) % world.length][(s + world[reihe].length)
					% world[reihe].length] == 1) {
				nachbarn++;
			}
		}
		if (world[reihe][(spalte - 1 + world[reihe].length)
				% world[reihe].length] == 1) {
			nachbarn++;
		}
		if (world[reihe][(spalte + 1) % world[reihe].length] == 1) {
			nachbarn++;
		}
		for (int s = spalte - 1; s <= spalte + 1; s++) {
			if (world[(reihe + 1) % world.length][(s + world[reihe].length)
					% world[reihe].length] == 1) {
				nachbarn++;
			}
		}
		return nachbarn;
	}
}
